import datetime
BIGDICT = {'Alcestis': 'mantilla',
 'arraignments': True,
 'bosh': 6909,
 'chant': 6785,
 'garbanzo': {'Armenian': True,
              'benign': {'easygoing': {'basted': 'counterpoint',
                                       'nonpolluting': 'Gretchen',
                                       'ointment': {'Talleyrand': False,
                                                    'comparably': 'hedgehog',
                                                    'goiter': 6895,
                                                    'medalist': {},
                                                    'pinked': False},
                                       'replication': datetime.datetime(1918, 3, 3, 8, 35),
                                       'ripsaw': 4316},
                         'interaction': datetime.datetime(1980, 7, 23, 8, 6),
                         'renaissances': datetime.datetime(2013, 1, 22, 18, 59),
                         'revision': True,
                         'substantiates': {'Pepsi': True,
                                           'affords': 5854,
                                           'buckskins': 'legalize',
                                           'syphilis': False,
                                           'underdeveloped': True}},
              'pennants': {'armadillo': {'Condillac': 'Equuleus',
                                         'Orly': datetime.datetime(1911, 1, 17, 4, 24),
                                         'civvies': 9109,
                                         'hilly': datetime.datetime(2013, 1, 22, 18, 59),
                                         'nurture': 1718},
                           'comers': 322,
                           'furtherance': {'Brahe': datetime.datetime(1980, 7, 23, 8, 6),
                                           'Whitehead': False,
                                           'joysticks': {'drinker': 6444,
                                                         'gamed': 'hosing',
                                                         'larkspur': 3430,
                                                         'prepaid': 'unspoiled',
                                                         'stockpile': datetime.datetime(1923, 6, 2, 15, 5)},
                                           'snarls': 'meditates',
                                           'studding': {'Botticelli': 8214,
                                                        'Uruguayans': 830,
                                                        'illicitness': 7397,
                                                        'rosin': False,
                                                        'underskirts': False}},
                           'immodestly': 4678,
                           'kleptomania': 'pounced'},
              'sachet': {'bankrolls': True,
                         'breeding': 9051,
                         'failure': datetime.datetime(1975, 7, 22, 4, 44),
                         'imposed': 'avenged',
                         'undeceived': {'festal': 3225,
                                        'intermezzi': 1484,
                                        'poor': False,
                                        'reprinted': datetime.datetime(1918, 3, 3, 8, 35),
                                        'sleepwalkers': 'embarrassing'}},
              'tryst': True},
 'inaugural': {'Horacio': datetime.datetime(1923, 6, 2, 15, 5),
               'Utopians': True,
               'chimeras': datetime.datetime(1967, 3, 1, 13, 42),
               'domed': 3535,
               'member': False},
 'lava': 'Domingo',
 'potash': 7166,
 'spoils': False,
 'subteens': False}

HUGEDICT = {'Agnes': 9671,
 'Alcuin': False,
 'Borges': [{'cokes': [],
             'contenting': {'blankly': {'Brownian': 2967,
                                        'bondage': 'Brant',
                                        'clubfoot': [],
                                        'hemophiliacs': 8448,
                                        'superstitious': datetime.datetime(1995, 1, 23, 9, 59)},
                            'gofers': datetime.datetime(1967, 3, 1, 13, 42),
                            'lifeblood': 'feisty',
                            'tomboy': 'filibusters',
                            'translated': []},
             'indifference': False,
             'mania': datetime.datetime(1987, 2, 2, 8, 31),
             'sweetly': True},
            'assessment'],
 'Braille': 'audio',
 'Cancer': 'reoccurs',
 'Devon': 'substituting',
 'Gilead': [],
 'Harry': 'seated',
 'Herschel': 'jack',
 'Horne': 6853,
 'Levine': datetime.datetime(1918, 3, 3, 8, 35),
 'Lugosi': {'accumulating': [False,
                             {'Tomsk': [True],
                              'forsakes': 7915,
                              'minuscules': 6354,
                              'petioles': False,
                              'waterside': 7134}],
            'depreciating': 3519,
            'fogey': 920,
            'outbalancing': 'thoughts',
            'porpoising': 3589},
 'Max': datetime.datetime(2005, 7, 13, 17, 18),
 'Mountbatten': 2791,
 'Oprah': False,
 'Peck': False,
 'Percival': [False,
              [datetime.datetime(1967, 3, 1, 13, 42),
               269,
               True,
               4595,
               {'Ovid': {'carpentered': 3486,
                         'chessman': [],
                         'exertion': 6442,
                         'masculine': 5995,
                         'unscrewed': 'otiose'},
                'Thaddeus': 'Botswana',
                'acknowledging': 5445,
                'dipstick': 'shimmered',
                'involuntarily': True}]],
 'Rhodes': 'spliced',
 'Rick': 3390,
 'Saskatoon': [],
 'Sudetenland': {'Brinkley': 2652,
                 'Nestle': {'agilely': 6234,
                            'execrate': datetime.datetime(2005, 7, 13, 17, 18),
                            'fittingly': datetime.datetime(1923, 6, 2, 15, 5),
                            'infomercials': datetime.datetime(1912, 1, 9, 15, 4),
                            'sarcastically': {'Verona': {'functions': False,
                                                         'hastiest': 8998,
                                                         'push': datetime.datetime(1936, 12, 16, 10, 38),
                                                         'sailors': {},
                                                         'smother': 'downpours'},
                                              'fiendishly': 6242,
                                              'gallstones': [[],
                                                             datetime.datetime(1967, 3, 1, 13, 42),
                                                             {},
                                                             False],
                                              'riles': datetime.datetime(1949, 6, 25, 23, 50),
                                              'vegetable': 'nursemaid'}},
                 'dauntlessly': [True,
                                 [],
                                 {'cask': 6334,
                                  'gamuts': {'Java': 8903,
                                             'Vilma': 7984,
                                             'god': {},
                                             'provisions': [],
                                             'shredded': []},
                                  'housings': 'Australasia',
                                  'stooping': [True,
                                               'stirrers',
                                               {},
                                               False,
                                               {}],
                                  'tied': True}],
                 'gruffly': 1466,
                 'roaches': 5433},
 'Veda': 'enterprise',
 'Vesalius': datetime.datetime(2011, 11, 1, 16, 26),
 'Vietnam': 3655,
 'Wilberforce': False,
 'Wolfe': False,
 'Wuhan': 'shippers',
 'Zeke': {'abusive': {'betoken': 1922,
                      'eigenvalues': {'Nebraskan': None,
                                      'chortles': datetime.datetime(1936, 12, 16, 10, 38),
                                      'engrossed': True,
                                      'harridan': {'Roquefort': [],
                                                   'canvased': [],
                                                   'reproduce': datetime.datetime(2005, 7, 13, 17, 18),
                                                   'spacesuit': False,
                                                   'watermarking': datetime.datetime(1918, 3, 3, 8, 35)},
                                      'recommences': None},
                      'fevers': datetime.datetime(1914, 5, 17, 9, 37),
                      'miff': [[2624,
                                datetime.datetime(1995, 1, 23, 9, 59),
                                430],
                               {'Hay': [],
                                'absolving': {},
                                'bibliography': [],
                                'cantons': datetime.datetime(2005, 7, 13, 17, 18),
                                'necking': datetime.datetime(1975, 7, 22, 4, 44)},
                               [[], {}],
                               'tincturing',
                               6372],
                      'subjugates': 3413},
          'basely': datetime.datetime(1949, 6, 25, 23, 50),
          'chiselers': {'Memphis': [False,
                                    True,
                                    [datetime.datetime(1918, 3, 3, 8, 35),
                                     {}]],
                        'incessantly': {'excrement': datetime.datetime(1923, 6, 2, 15, 5),
                                        'opaquing': 'deviltry',
                                        'patriotically': datetime.datetime(1936, 12, 16, 10, 38),
                                        'robbing': False,
                                        'stanching': False},
                        'nightgown': datetime.datetime(1912, 1, 9, 15, 4),
                        'papyrus': {'Malachi': False,
                                    'ditch': False,
                                    'flypapers': datetime.datetime(2011, 11, 1, 16, 26),
                                    'nous': None,
                                    'swirled': datetime.datetime(1980, 7, 23, 8, 6)},
                        'shirred': False},
          'jot': 'wrestles',
          'squeezer': ['orbs']},
 'Zomba': None,
 'abducts': {'Pei': datetime.datetime(1995, 1, 23, 9, 59),
             'floorboard': 'curvy',
             'gulling': 'inhaling',
             'pungently': True,
             'vodka': 'heifers'},
 'abortionists': datetime.datetime(2011, 11, 1, 16, 26),
 'accomplishing': True,
 'accounting': [7812, 6998],
 'adapter': {'brainwashed': [datetime.datetime(1967, 3, 1, 13, 42),
                             None,
                             'confrontations'],
             'earthquake': [{'bus': [5385, 'unburdening', [], {}],
                             'caustic': 'peritonitis',
                             'eulogies': 9254,
                             'intimation': [{},
                                            datetime.datetime(2005, 7, 13, 17, 18),
                                            datetime.datetime(1936, 12, 16, 10, 38),
                                            False],
                             'shakiest': ['permanence', 'madrigal']},
                            6064],
             'indomitably': False,
             'liar': ['mesmerized'],
             'sectioning': [['Parsi', datetime.datetime(1983, 6, 27, 3, 56)],
                            datetime.datetime(1949, 6, 25, 23, 50),
                            datetime.datetime(1980, 7, 23, 8, 6),
                            5570,
                            datetime.datetime(1923, 6, 2, 15, 5)]},
 'aesthetes': {'berths': True,
               'massages': False,
               'method': datetime.datetime(1918, 3, 3, 8, 35),
               'pealing': 4645,
               'synod': []},
 'alighted': False,
 'angst': datetime.datetime(1912, 1, 9, 15, 4),
 'apprenticeship': [3713,
                    {'braked': 6364,
                     'chimpanzees': datetime.datetime(1995, 1, 23, 9, 59),
                     'floggings': 'yearnings',
                     'notoriety': datetime.datetime(1980, 7, 23, 8, 6),
                     'tranquilest': [6857]},
                    datetime.datetime(1975, 7, 22, 4, 44)],
 'arcing': 'Holmes',
 'ashore': False,
 'backward': 6032,
 'behoove': 4094,
 'bigmouths': 'scrapers',
 'blinded': 'nonresident',
 'bombshell': 1252,
 'both': 'arrow',
 'bouncy': None,
 'bunked': datetime.datetime(1995, 1, 23, 9, 59),
 'buy': [datetime.datetime(1995, 1, 23, 9, 59)],
 'canonical': {'backfields': 'cosignatories',
               'fain': datetime.datetime(1983, 6, 27, 3, 56),
               'frighting': datetime.datetime(1930, 12, 12, 3, 58),
               'shimmy': 5768,
               'spastics': datetime.datetime(1911, 1, 17, 4, 24)},
 'canonizations': 'battle',
 'catalogs': False,
 'categorization': False,
 'charade': 7048,
 'cholesterol': datetime.datetime(1983, 6, 27, 3, 56),
 'collected': [],
 'communicated': [datetime.datetime(1975, 7, 22, 4, 44),
                  True,
                  7047,
                  {'aspirating': 4305,
                   'conjoining': 'privy',
                   'damask': 'impala',
                   'hearts': [],
                   'unworthy': 7095}],
 'competences': 2791,
 'confederacy': 'polliwogs',
 'conjecture': ['salubrious', 5367, 9920],
 'contaminant': datetime.datetime(1911, 1, 17, 4, 24),
 'contravention': datetime.datetime(1949, 6, 25, 23, 50),
 'coping': [9861, False, 3763],
 'cornerstones': {'evildoers': ['chowder', 4864],
                  'passageway': datetime.datetime(2011, 11, 1, 16, 26),
                  'pastry': {'fetishists': [687, None, 5846],
                             'giblets': [6903,
                                         [[],
                                          [],
                                          'corporals',
                                          'glinted',
                                          {}],
                                         [datetime.datetime(1936, 12, 16, 10, 38),
                                          1666,
                                          True,
                                          {},
                                          False],
                                         5831,
                                         8925],
                             'glorifying': 'staked',
                             'maestro': 'amalgam',
                             'villainy': 'ungodlier'},
                  'perfidious': [5013,
                                 {'aromas': None,
                                  'boilerplate': {'foreseeable': datetime.datetime(2011, 11, 1, 16, 26),
                                                  'kidnapping': True,
                                                  'monotheistic': 'repeals',
                                                  'pejoratives': [],
                                                  'thwacked': 'Nicaea'},
                                  'casseroles': True,
                                  'indicted': {'Gaines': 'liaises',
                                               'manse': [],
                                               'marauded': 'Praetorian',
                                               'prefix': datetime.datetime(1936, 12, 16, 10, 38),
                                               'sweetbriar': 'Barbuda'},
                                  'transgresses': [False]},
                                 [datetime.datetime(1987, 2, 2, 8, 31),
                                  6653,
                                  {'Dachau': [],
                                   'anger': datetime.datetime(1918, 3, 3, 8, 35),
                                   'expatiate': True,
                                   'portioning': False,
                                   'swooning': 8374},
                                  [],
                                  6871],
                                 8436],
                  'retirement': 7577},
 'corollaries': {'Ger': datetime.datetime(1923, 6, 2, 15, 5),
                 'downloads': 9182,
                 'hospitable': datetime.datetime(1949, 6, 25, 23, 50),
                 'playfulness': 5617,
                 'urethra': datetime.datetime(1914, 5, 17, 9, 37)},
 'corrections': 8263,
 'courtliness': 'poorer',
 'cozy': False,
 'cuddlier': [datetime.datetime(1966, 7, 22, 17, 30), 'tutored'],
 'cumming': 'ghostlier',
 'darnedest': False,
 'declaims': {'analyticalally': datetime.datetime(1980, 7, 23, 8, 6),
              'availing': {'calumniated': datetime.datetime(2005, 7, 13, 17, 18),
                           'chaffs': {'Cadillac': [{}, 'Tucker', {}],
                                      'Pecos': 'Sarto',
                                      'caw': {'disrespectfully': [],
                                              'maypole': [],
                                              'stagnant': {},
                                              'unite': {},
                                              'zap': {}},
                                      'javelin': [{}, 8780],
                                      'sycamore': 7456},
                           'enchanted': False,
                           'manholes': False,
                           'miss': [{'Trobriand': {},
                                     'dialects': 'preoccupation',
                                     'elide': datetime.datetime(1980, 7, 23, 8, 6),
                                     'fabricate': {},
                                     'smackers': datetime.datetime(1967, 3, 1, 13, 42)},
                                    datetime.datetime(1983, 6, 27, 3, 56),
                                    5817,
                                    [True,
                                     datetime.datetime(1918, 3, 3, 8, 35),
                                     {},
                                     'postponement',
                                     8970],
                                    False]},
              'crackpot': 2879,
              'shufflers': [{'besieged': {'fixity': 9979,
                                          'heave': datetime.datetime(1912, 1, 9, 15, 4),
                                          'overseers': datetime.datetime(1936, 12, 16, 10, 38),
                                          'plowmen': True,
                                          'triptychs': 1072},
                             'central': 5135,
                             'faultier': 'sweetens',
                             'maturer': ['opuses', []],
                             'proverbially': 2744},
                            True],
              'spoon': 4998},
 'decomposes': [False],
 'dehumanize': True,
 'depopulating': 'Vulgate',
 'deride': datetime.datetime(1975, 7, 22, 4, 44),
 'destructs': 'unfathomable',
 'detonators': True,
 'disabused': {'Dumpster': {'Iva': ['freeloaded',
                                    True,
                                    ['misplayed',
                                     {},
                                     [],
                                     False,
                                     datetime.datetime(1918, 3, 3, 8, 35)],
                                    5169,
                                    False],
                            'faker': 'gunning',
                            'scorecard': {'Brittany': 5579,
                                          'maltreat': False,
                                          'promptings': {'furze': 'Karenina',
                                                         'importuning': {},
                                                         'litigiousness': 'tenderfoots',
                                                         'sensitives': 'reassuringly',
                                                         'welshing': 3201},
                                          'selectors': datetime.datetime(2013, 1, 22, 18, 59),
                                          'spray': datetime.datetime(1914, 5, 17, 9, 37)},
                            'sleepwalking': {'asphalting': 6997,
                                             'dish': [],
                                             'drain': 'surfeiting',
                                             'fleeces': {'admired': 2124,
                                                         'disappoint': 'earthly',
                                                         'farina': False,
                                                         'klutzier': 'nightfall',
                                                         'sublimation': {}},
                                             'sleepier': True},
                            'split': {'Rather': [{},
                                                 'transference',
                                                 True,
                                                 datetime.datetime(1914, 5, 17, 9, 37)],
                                      'Ymir': False,
                                      'antenna': True,
                                      'basal': {'Brahmin': {},
                                                'beneficent': datetime.datetime(1936, 12, 16, 10, 38),
                                                'depositions': datetime.datetime(1949, 6, 25, 23, 50),
                                                'scums': {},
                                                'uphills': None},
                                      'commandoes': 'lipreading'}},
               'Glaswegian': 'juicier',
               'collaborate': False,
               'recessed': {'astronaut': datetime.datetime(1975, 7, 22, 4, 44),
                            'laughter': [datetime.datetime(1983, 6, 27, 3, 56),
                                         8038,
                                         True,
                                         {'Jose': [],
                                          'ballsier': {},
                                          'casks': {},
                                          'harnessing': 'artificial',
                                          'integrated': 'alphabet'},
                                         True],
                            'ornate': {'Mondrian': {'Corfu': True,
                                                    'blithely': {},
                                                    'essence': 7905,
                                                    'lithe': datetime.datetime(1930, 12, 12, 3, 58),
                                                    'obscurity': {}},
                                       'ayes': datetime.datetime(1912, 1, 9, 15, 4),
                                       'homeopathy': True,
                                       'neglect': {'Abrams': False,
                                                   'gook': True,
                                                   'legitimizes': False,
                                                   'pervades': {},
                                                   'tillable': datetime.datetime(1923, 6, 2, 15, 5)},
                                       'slaver': [521]},
                            'shaken': 3158,
                            'workers': 3137},
               'silvery': datetime.datetime(1923, 6, 2, 15, 5)},
 'disco': True,
 'discrimination': True,
 'displeases': [],
 'doyens': [[2614, False, 9485, datetime.datetime(1983, 6, 27, 3, 56)],
            {'Church': datetime.datetime(1923, 6, 2, 15, 5),
             'Devin': {'Hispanic': [558, [], []],
                       'asylums': 'denotes',
                       'brusquer': {'battlefields': 9566,
                                    'drape': 1006,
                                    'flagellate': 'rued',
                                    'logic': True,
                                    'randomized': {}},
                       'incursions': 'briefcases',
                       'initiates': 'Brahmanisms'},
             'confided': {'approbation': 7365,
                          'duded': 'visualizing',
                          'enmesh': True,
                          'separately': datetime.datetime(1967, 3, 1, 13, 42),
                          'xylophone': 7646},
             'mugged': False,
             'reminiscence': True},
            [{'Cohan': False,
              'Mesmer': datetime.datetime(1966, 7, 22, 17, 30),
              'coke': 803,
              'cozily': True,
              'extort': 'entirely'},
             [True, [2782, 8506], 'scintillation', 3006],
             5032,
             {'Grumman': datetime.datetime(1936, 12, 16, 10, 38),
              'cloaks': 'adverbial',
              'culprit': 8740,
              'egocentrics': datetime.datetime(1980, 7, 23, 8, 6),
              'objectionable': False}],
            {'Belgrade': [],
             'Lagos': True,
             'agribusinesses': 'mercerize',
             'enamor': False,
             'exigency': 'psychogenic'},
            {'divinely': {'coiled': True,
                          'jinns': [{}],
                          'multitudes': [[],
                                         datetime.datetime(2005, 7, 13, 17, 18),
                                         [],
                                         []],
                          'reactionaries': 8034,
                          'stairwells': {'gasping': datetime.datetime(1936, 12, 16, 10, 38),
                                         'grouted': datetime.datetime(1914, 5, 17, 9, 37),
                                         'paraffin': {},
                                         'redheaded': datetime.datetime(1949, 6, 25, 23, 50),
                                         'remakes': datetime.datetime(1967, 3, 1, 13, 42)}},
             'easily': 'backlogged',
             'lauds': 3708,
             'philharmonic': {'butterfingers': 'Jordan',
                              'cleavers': 'leash',
                              'gloamings': datetime.datetime(1975, 7, 22, 4, 44),
                              'lament': {'Moldavia': [],
                                         'Moore': {},
                                         'newsboys': True,
                                         'parapets': 'revolutionary',
                                         'savored': False},
                              'surveyor': 3300},
             'wormiest': {'draconian': 'evillest',
                          'quoiting': 1886,
                          'restores': {'constricted': datetime.datetime(1980, 7, 23, 8, 6),
                                       'enjoyed': 'remains',
                                       'grammars': datetime.datetime(1987, 2, 2, 8, 31),
                                       'hateful': 'brotherliness',
                                       'temporary': False},
                          'sputters': [[],
                                       [],
                                       True,
                                       'lipsticks',
                                       datetime.datetime(1923, 6, 2, 15, 5)],
                          'valedictory': 'correlated'}}],
 'dumfounding': datetime.datetime(1975, 7, 22, 4, 44),
 'earthly': 'colonial',
 'edibles': 'enraging',
 'encored': datetime.datetime(1930, 12, 12, 3, 58),
 'extraordinarily': 'Kirsten',
 'eyepiece': True,
 'facsimiled': 'hermetic',
 'fanzine': [[{'abnegated': 'unholiest',
               'banning': 'indicatives',
               'cameras': False,
               'insecticide': [{}, [], 2702],
               'yielded': {'bloodhounds': 4940,
                           'gendarmes': [],
                           'isinglass': [],
                           'ridging': False,
                           'stripling': []}},
              False,
              4290,
              3428],
             True,
             datetime.datetime(1966, 7, 22, 17, 30),
             8379,
             'grain'],
 'filching': [datetime.datetime(1930, 12, 12, 3, 58),
              False,
              2491,
              {'brutalize': {'Polly': False,
                             'Toledo': 3889,
                             'gladiolus': 'rundowns',
                             'heartiest': 6050,
                             'spite': {'Pygmies': {},
                                       'axiomatic': {},
                                       'degrees': 347,
                                       'dismember': 'mayhem',
                                       'retrospect': {}}},
               'docket': datetime.datetime(1936, 12, 16, 10, 38),
               'purplish': datetime.datetime(1967, 3, 1, 13, 42),
               'scientists': [],
               'spates': 'patriarch'},
              {'Manet': 5895,
               'bashed': 9585,
               'coveralls': {'frown': 2119,
                             'heathens': False,
                             'hovel': 'gnaws',
                             'slovenliness': [],
                             'unorthodox': ['riming',
                                            'mortgager',
                                            'encamps',
                                            False,
                                            []]},
               'envisioning': False,
               'undo': datetime.datetime(1980, 7, 23, 8, 6)}],
 'fillips': datetime.datetime(1914, 5, 17, 9, 37),
 'filter': datetime.datetime(1966, 7, 22, 17, 30),
 'fleeting': [[['proceeds'], 'northwestern'], 'gray'],
 'flicking': True,
 'floodlit': True,
 'flunking': True,
 'forged': 8401,
 'fryer': datetime.datetime(1975, 7, 22, 4, 44),
 'garbageman': 6151,
 'gigglier': {'Reinaldo': {'breached': datetime.datetime(1930, 12, 12, 3, 58),
                           'kidney': 7671,
                           'likely': datetime.datetime(2005, 7, 13, 17, 18),
                           'orphans': {'Handel': {'Mitch': 'bowdlerized',
                                                  'Okefenokee': 9560,
                                                  'auto': True,
                                                  'hurtful': [],
                                                  'rediscovery': 'deplanes'},
                                       'creativity': {'Ouagadougou': 5593,
                                                      'Stirling': [],
                                                      'filet': None,
                                                      'posts': False,
                                                      'stranglehold': {}},
                                       'feral': 'Bastille',
                                       'gerrymandering': 'Udall',
                                       'xenon': datetime.datetime(1911, 1, 17, 4, 24)},
                           'yawed': True},
              'clerk': [[]],
              'fetuses': 2858,
              'happen': True,
              'sappy': datetime.datetime(1966, 7, 22, 17, 30)},
 'goggling': 7184,
 'groupie': {'Marcia': {'baboons': 4365,
                        'jobless': [datetime.datetime(2005, 7, 13, 17, 18),
                                    'implacable',
                                    {'Janjaweed': [],
                                     'Morita': 7946,
                                     'bouillabaisses': False,
                                     'geeing': datetime.datetime(1995, 1, 23, 9, 59),
                                     'overwrites': []}],
                        'propound': 976,
                        'travestied': True,
                        'vamped': datetime.datetime(1995, 1, 23, 9, 59)},
             'promos': ['curdling',
                        [datetime.datetime(1930, 12, 12, 3, 58),
                         datetime.datetime(1914, 5, 17, 9, 37),
                         {'conspirators': datetime.datetime(1923, 6, 2, 15, 5),
                          'flummoxes': 'hoots',
                          'masthead': 'desist',
                          'segregates': 'encumbered',
                          'weavers': 'glades'},
                         datetime.datetime(1914, 5, 17, 9, 37),
                         [3120]],
                        1007,
                        datetime.datetime(1987, 2, 2, 8, 31)],
             'slingshot': [],
             'spelling': datetime.datetime(1912, 1, 9, 15, 4),
             'tedious': {'Albee': 'vamped',
                         'Byelorussia': datetime.datetime(1914, 5, 17, 9, 37),
                         'fishtailed': False,
                         'lagging': True,
                         'megalopolis': datetime.datetime(1911, 1, 17, 4, 24)}},
 'hectoring': datetime.datetime(1987, 2, 2, 8, 31),
 'holdouts': ['pinions',
              {'Schiller': True,
               'dairyman': {'batty': 'ironies',
                            'enthrones': 'slicking',
                            'implementation': {'Fredrick': [],
                                               'Irma': True,
                                               'Kurile': datetime.datetime(1911, 1, 17, 4, 24),
                                               'daintiness': datetime.datetime(2013, 1, 22, 18, 59),
                                               'invaluable': 'woodpile'},
                            'obelisks': 3312,
                            'sleigh': datetime.datetime(1930, 12, 12, 3, 58)},
               'mainstreamed': [3505],
               'merry': datetime.datetime(1983, 6, 27, 3, 56),
               'reek': True},
              False,
              [{'Muawiya': 8138,
                'dosed': 'nonprescription',
                'ports': 'immunizing',
                'streptomycin': True,
                'trusty': {'Cornell': 'hushes',
                           'allergists': datetime.datetime(1930, 12, 12, 3, 58),
                           'clinging': {},
                           'mainsprings': 'bequeathed',
                           'tunnelled': datetime.datetime(1911, 1, 17, 4, 24)}},
               {'crimsons': datetime.datetime(1949, 6, 25, 23, 50),
                'grousing': datetime.datetime(1967, 3, 1, 13, 42),
                'nominee': 'overdosing',
                'pants': {'anecdote': False,
                          'cuttings': 'seeded',
                          'dolphin': 'purblind',
                          'recessives': 7595,
                          'satirist': 2061},
                'shinned': datetime.datetime(2013, 1, 22, 18, 59)},
               'cumbersome',
               {'chartreuse': True,
                'demoralizing': 'centipedes',
                'keys': {'Bahamas': 'soprano',
                         'Cepheus': [],
                         'hurriedly': 'deferentially',
                         'snowfall': 1276,
                         'unconventional': True},
                'kittens': 2653,
                'waxed': 'enthrals'},
               True],
              [{'aligned': True,
                'articulated': [{}, 6482, 'hybrid'],
                'canticles': 2660,
                'evening': [],
                'villainies': 1006},
               3786,
               [8586,
                'pioneers',
                {'abortionists': True,
                 'epithets': 'Fran',
                 'gangly': [],
                 'largess': [],
                 'pretending': 3879}],
               1473]],
 'horseradish': 'joy',
 'hostilities': {'Casals': {'clipt': datetime.datetime(1966, 7, 22, 17, 30),
                            'libations': 1207,
                            'servomechanisms': False,
                            'stopcock': 'digressed',
                            'usherette': True},
                 'Lenten': 2397,
                 'conditionally': True,
                 'meritocracies': {'attic': [],
                                   'eclipses': datetime.datetime(1987, 2, 2, 8, 31),
                                   'however': 'Mt',
                                   'manages': 'whirs',
                                   'nights': [[datetime.datetime(2005, 7, 13, 17, 18)],
                                              True,
                                              8485]},
                 'tuba': 3140},
 'humbler': ['soundproofs'],
 'humps': [3576,
           False,
           [[],
            datetime.datetime(1987, 2, 2, 8, 31),
            6898,
            datetime.datetime(1911, 1, 17, 4, 24),
            datetime.datetime(1995, 1, 23, 9, 59)]],
 'huskiness': 150,
 'imbalanced': 5365,
 'imperturbable': 1817,
 'incrusting': 3456,
 'intensifier': datetime.datetime(1966, 7, 22, 17, 30),
 'irritatingly': False,
 'itineraries': datetime.datetime(1912, 1, 9, 15, 4),
 'jeremiad': [5655],
 'judgements': datetime.datetime(1966, 7, 22, 17, 30),
 'knife': 2500,
 'lactates': {'amounting': 'interconnects',
              'awkwardest': True,
              'cistern': 3703,
              'gearbox': datetime.datetime(2013, 1, 22, 18, 59),
              'pastured': None},
 'laundering': datetime.datetime(2005, 7, 13, 17, 18),
 'litanies': 'Schroeder',
 'lumpiness': datetime.datetime(1983, 6, 27, 3, 56),
 'malpractice': False,
 'malt': [datetime.datetime(1995, 1, 23, 9, 59)],
 'menopausal': 'distrust',
 'middleman': {'Jerrold': 8321,
               'diddling': 'completer',
               'exegesis': False,
               'intransitives': 'sootier',
               'securing': False},
 'mil': {'chopstick': 'tingled',
         'controversy': datetime.datetime(1911, 1, 17, 4, 24),
         'gees': datetime.datetime(1911, 1, 17, 4, 24),
         'realest': 5065,
         'tyrants': True},
 'miscued': 2061,
 'misfires': datetime.datetime(1995, 1, 23, 9, 59),
 'modeling': [255,
              {'officers': {'afflicted': True,
                            'bamboozle': {'Cabot': {},
                                          'Li': 'thunderclouds',
                                          'grubbing': datetime.datetime(2013, 1, 22, 18, 59),
                                          'lowly': [],
                                          'reminisces': datetime.datetime(1918, 3, 3, 8, 35)},
                            'dialog': [True, [], {}, 2935],
                            'harmonization': datetime.datetime(1923, 6, 2, 15, 5),
                            'octettes': True},
               'scoops': 2437,
               'socializes': 149,
               'tropical': True,
               'vs': [6921, 7216, 'antiphonals', [{}, []]]}],
 'moonlighted': 2466,
 'mushes': 'Ikea',
 'muskrats': 'trinkets',
 'necked': 'aggrieve',
 'nominally': datetime.datetime(1912, 1, 9, 15, 4),
 'nullify': datetime.datetime(1923, 6, 2, 15, 5),
 'occult': 'molders',
 'oculars': {'Fridays': [False,
                         'telephone',
                         datetime.datetime(2005, 7, 13, 17, 18),
                         'creditable',
                         'smothered'],
             'Google': datetime.datetime(1995, 1, 23, 9, 59),
             'broader': [False,
                         True,
                         {'abated': False,
                          'biscuits': [datetime.datetime(2011, 11, 1, 16, 26),
                                       'slipknots'],
                          'nigh': 'Angolan',
                          'precursor': datetime.datetime(1912, 1, 9, 15, 4),
                          'tiptoeing': datetime.datetime(1967, 3, 1, 13, 42)},
                         [False, False]],
             'linkups': {'chessboards': [{'Omahas': True,
                                          'exclusively': False,
                                          'index': [],
                                          'misgovern': {},
                                          'unscrambles': False},
                                         'undue'],
                         'dazzled': datetime.datetime(1995, 1, 23, 9, 59),
                         'hellholes': 4717,
                         'tinted': 'erstwhile',
                         'travesties': 5393},
             'masts': False},
 'ordains': 'Caitlin',
 'oversimplifies': 'healthy',
 'parcelled': [],
 'patsies': [[],
             {'afforest': [],
              'incident': {'Virgil': datetime.datetime(1912, 1, 9, 15, 4),
                           'contents': 'row',
                           'dashiki': datetime.datetime(1912, 1, 9, 15, 4),
                           'gonorrhea': 8686,
                           'screeches': True},
              'overwhelm': datetime.datetime(2011, 11, 1, 16, 26),
              'parliamentarian': {'astrophysics': 5675,
                                  'enquired': True,
                                  'outsizes': {'Huff': datetime.datetime(1911, 1, 17, 4, 24),
                                               'crews': datetime.datetime(1930, 12, 12, 3, 58),
                                               'synonym': datetime.datetime(1983, 6, 27, 3, 56),
                                               'wangles': 4132,
                                               'weighs': {}},
                                  'whatsoever': [{},
                                                 'paranoids',
                                                 datetime.datetime(2011, 11, 1, 16, 26)],
                                  'yodellers': [197,
                                                [],
                                                False,
                                                datetime.datetime(1987, 2, 2, 8, 31)]},
              'triptychs': False},
             [],
             'Novembers'],
 'pigsties': ['delectation',
              {'arresting': False,
               'inner': 3268,
               'medications': {'beefs': 'prism',
                               'judgmental': [{}, {}],
                               'moguls': {'Brecht': 4463,
                                          'absorb': [],
                                          'trims': {},
                                          'tuns': {},
                                          'underestimating': {}},
                               'riles': 2493,
                               'shuttlecocked': True},
               'posthumously': 'opulent',
               'shoot': False},
              5265,
              {'Mirfak': 'gouger',
               'amaranths': datetime.datetime(1995, 1, 23, 9, 59),
               'email': datetime.datetime(2013, 1, 22, 18, 59),
               'engorge': 6681,
               'fetid': datetime.datetime(1987, 2, 2, 8, 31)},
              5694],
 'piking': False,
 'plants': True,
 'players': False,
 'plod': datetime.datetime(1918, 3, 3, 8, 35),
 'pointless': {'Lot': 'Cezanne',
               'bonnets': True,
               'glider': datetime.datetime(1949, 6, 25, 23, 50),
               'playback': {'birdbath': 1180,
                            'cabarets': 2531,
                            'oversupplied': 'viscountesses',
                            'protoplasm': 7240,
                            'wuss': 'wryly'},
               'stallion': {'pans': datetime.datetime(2005, 7, 13, 17, 18),
                            'photosynthesis': False,
                            'presidential': True,
                            'sufficiency': 'liqueurs',
                            'traders': datetime.datetime(1918, 3, 3, 8, 35)}},
 'polarization': 4126,
 'polliwog': ['humanism',
              7118,
              [datetime.datetime(2005, 7, 13, 17, 18), 6143]],
 'possession': 8822,
 'quash': [datetime.datetime(1987, 2, 2, 8, 31), 6777],
 'razzed': None,
 'reactivates': [datetime.datetime(1983, 6, 27, 3, 56)],
 'realities': 'refundable',
 'regimentation': True,
 'riddles': False,
 'romanced': 'extinguishers',
 'salesgirl': 3437,
 'saut\xc3\xa9ing': 'preoccupation',
 'scone': 8713,
 'senator': 'interpreted',
 'skepticism': {'Dec': 6776,
                'antiquary': ['setting',
                              datetime.datetime(1936, 12, 16, 10, 38),
                              {'Lemaitre': [8874, 9306],
                               'accentuate': {'flashlight': datetime.datetime(1966, 7, 22, 17, 30),
                                              'flooding': datetime.datetime(1936, 12, 16, 10, 38),
                                              'ladies': [],
                                              'ravening': [],
                                              'versification': {}},
                               'movables': datetime.datetime(1995, 1, 23, 9, 59),
                               'murmur': datetime.datetime(1949, 6, 25, 23, 50),
                               'parenthood': ['submerging', False]}],
                'grandparent': {'Vilma': {'Gallup': {'altimeter': 'overrode',
                                                     'burnouses': [],
                                                     'interloper': False,
                                                     'partitioning': datetime.datetime(1966, 7, 22, 17, 30),
                                                     'ukeleles': {}},
                                          'Gamay': False,
                                          'dashboard': 'wises',
                                          'mastoid': True,
                                          'terrible': datetime.datetime(1987, 2, 2, 8, 31)},
                                'agilest': datetime.datetime(1936, 12, 16, 10, 38),
                                'farrows': [False, 'Euterpe'],
                                'principals': {'bolts': datetime.datetime(1936, 12, 16, 10, 38),
                                               'leverages': 327,
                                               'rescuers': True,
                                               'scurrilous': {'castrating': False,
                                                              'charwomen': 6469,
                                                              'disuniting': {},
                                                              'ruffians': 'bunnies',
                                                              'statehood': datetime.datetime(1980, 7, 23, 8, 6)},
                                               'welching': 'disengagements'},
                                'sheepishness': [[1155, [], {}], 9034]},
                'habitu\xc3\xa9': {'Blacks': 'thymi',
                                   'Hans': False,
                                   'luminous': datetime.datetime(1914, 5, 17, 9, 37),
                                   'salubrious': 'contusions',
                                   'sonorous': 4393},
                'lucidness': {'Eyck': datetime.datetime(1949, 6, 25, 23, 50),
                              'bulging': [],
                              'feed': 'salmonellae',
                              'garrotes': datetime.datetime(1983, 6, 27, 3, 56),
                              'untested': None}},
 'slogging': datetime.datetime(2005, 7, 13, 17, 18),
 'smelliest': 8820,
 'smoker': 8780,
 'souvenir': ['regurgitation'],
 'sovereigns': 2534,
 'stormed': True,
 'stultified': [5535,
                'special',
                {'Berkshires': {'Nam': False,
                                'Weldon': {'Zachery': False,
                                           'betake': 6538,
                                           'cattleman': [],
                                           'milquetoasts': {},
                                           'undoubted': True},
                                'concreting': 'killed',
                                'cramps': [[],
                                           datetime.datetime(1912, 1, 9, 15, 4),
                                           {},
                                           False,
                                           {}],
                                'encumbrance': datetime.datetime(1966, 7, 22, 17, 30)},
                 'counsels': 'therapeutic',
                 'foregoing': 7634,
                 'linking': {'fructifies': {'Rushdie': False,
                                            'elided': 559,
                                            'endorsing': False,
                                            'nosediving': datetime.datetime(1987, 2, 2, 8, 31),
                                            'pivot': 'tarrier'},
                             'hairbreadth': datetime.datetime(1995, 1, 23, 9, 59),
                             'minimizing': {'Colosseum': True,
                                            'compacts': 'revolves',
                                            'piping': True,
                                            'repress': [],
                                            'violets': 'disassemble'},
                             'monaural': [3823, []],
                             'roiled': 6575},
                 'occasionally': ['balkier',
                                  datetime.datetime(1975, 7, 22, 4, 44),
                                  {'granulation': {},
                                   'groveling': 5952,
                                   'liquidized': True,
                                   'palsying': 'hems',
                                   'skid': datetime.datetime(2011, 11, 1, 16, 26)},
                                  [False, {}, 9829],
                                  True]}],
 'subscribed': datetime.datetime(1923, 6, 2, 15, 5),
 'success': {'Patricia': 2829,
             'conquistador': 'Ziegfeld',
             'indulges': True,
             'redrafts': {'ascertain': datetime.datetime(1967, 3, 1, 13, 42),
                          'divesting': {'alto': datetime.datetime(1949, 6, 25, 23, 50),
                                        'roadhouse': {'Bishop': True,
                                                      'allusive': datetime.datetime(1912, 1, 9, 15, 4),
                                                      'battened': 'overbooking',
                                                      'bloodthirstier': datetime.datetime(1995, 1, 23, 9, 59),
                                                      'decree': datetime.datetime(1923, 6, 2, 15, 5)},
                                        'sixties': 1285,
                                        'spool': None,
                                        'tinseling': True},
                          'instigated': 'sombrero',
                          'overspecialize': datetime.datetime(1983, 6, 27, 3, 56),
                          'seeds': 7127},
             'surfing': [8139, datetime.datetime(1930, 12, 12, 3, 58), 928]},
 'summered': True,
 'surrendering': {'crudit\xc3\xa9s': True,
                  'decrease': {'Cordoba': False,
                               'Plataea': False,
                               'Zairian': 'haves',
                               'debarks': datetime.datetime(1912, 1, 9, 15, 4),
                               'knocked': [4569, 5523, True, 8303, 5523]},
                  'prohibitory': {'Mach': True,
                                  'leakage': [['bone',
                                               datetime.datetime(1966, 7, 22, 17, 30),
                                               [],
                                               datetime.datetime(2011, 11, 1, 16, 26),
                                               {}],
                                              True,
                                              {'Fujitsu': False,
                                               'Morton': [],
                                               'bibulous': 'unjustifiable',
                                               'fractures': [],
                                               'grisly': {}},
                                              datetime.datetime(1911, 1, 17, 4, 24)],
                                  'mistime': 'daintiness',
                                  'probed': [],
                                  'urbaner': [{'Gucci': False,
                                               'Murat': 6605,
                                               'commerce': [],
                                               'growling': 3691,
                                               'nubile': 'maidenly'},
                                              {'Wassermann': False,
                                               'blackberry': 'publishes',
                                               'dissoluteness': True,
                                               'limier': 'drummer',
                                               'ponies': []},
                                              datetime.datetime(1949, 6, 25, 23, 50),
                                              ['nationwide', 'nets'],
                                              'dated']},
                  'waistline': 4120,
                  'zinced': 'dimwit'},
 'tanagers': None,
 'taxes': {'historically': [datetime.datetime(2013, 1, 22, 18, 59),
                            1129,
                            {'Grady': True,
                             'Mabel': 2253,
                             'adversary': datetime.datetime(1967, 3, 1, 13, 42),
                             'bares': False,
                             'caroms': 'entrepreneurial'},
                            {'disarm': {'Giza': [],
                                        'Mona': True,
                                        'Rapunzel': 'weltering',
                                        'radiologist': [],
                                        'raspberry': False},
                             'northerlies': {'Suleiman': 2825,
                                             'chicory': {},
                                             'feverishly': datetime.datetime(1995, 1, 23, 9, 59),
                                             'holsters': {},
                                             'threw': datetime.datetime(1987, 2, 2, 8, 31)},
                             'scale': [],
                             'stanch': 839,
                             'tune': 'evangelizing'},
                            True],
           'levering': datetime.datetime(1914, 5, 17, 9, 37),
           'teller': [{'Staples': 'danced',
                       'cocoon': 4437,
                       'secessionist': 6416,
                       'sways': 6023,
                       'venal': True},
                      [{'ambles': [],
                        'fetiches': 2096,
                        'secularize': {},
                        'sm\xc3\xb6rg\xc3\xa5sbord': datetime.datetime(1983, 6, 27, 3, 56),
                        'tartans': 34},
                       datetime.datetime(1911, 1, 17, 4, 24),
                       'sizzle',
                       8784]],
           'turret': 'moppets',
           'vacates': datetime.datetime(2011, 11, 1, 16, 26)},
 'teachable': 6100,
 'temporarily': None,
 'thoughtlessness': 2644,
 'totalled': 'extremer',
 'towheads': 5112,
 'traipsed': {'freeze': ['miscarried',
                         [['adjudication', [], 5849],
                          [datetime.datetime(1983, 6, 27, 3, 56), True],
                          6103,
                          [[], {}, None, True]],
                         False,
                         datetime.datetime(1918, 3, 3, 8, 35)],
              'hoses': False,
              'insinuate': 'handiest',
              'rethinks': 'fashion',
              'vibe': 'phished'},
 'twentieths': [],
 'unappetizing': datetime.datetime(1983, 6, 27, 3, 56),
 'unappreciated': True,
 'undernourished': [False,
                    [{'Diaspora': {'equivocating': 4363,
                                   'fingerings': 'Hannibal',
                                   'sorter': 'beatified',
                                   'tony': {},
                                   'variegate': True},
                      'Kurtis': 'adjudicated',
                      'chunks': {'approximately': 'winded',
                                 'brush': datetime.datetime(1918, 3, 3, 8, 35),
                                 'hack': datetime.datetime(1911, 1, 17, 4, 24),
                                 'spectacularly': datetime.datetime(2005, 7, 13, 17, 18),
                                 'tripe': 'untimelier'},
                      'effecting': 'bullfighters',
                      'loafer': {'Kentucky': datetime.datetime(1995, 1, 23, 9, 59),
                                 'Lenten': 462,
                                 'Tarkenton': [],
                                 'disappoints': {},
                                 'inked': 6087}}],
                    'wayfarer',
                    datetime.datetime(1949, 6, 25, 23, 50),
                    'goofing'],
 'unhealthful': 'toddling',
 'uninstaller': 'leach',
 'unquestionably': 'reprocesses',
 'unwieldiness': {'betrayer': [[{'jellybean': datetime.datetime(1966, 7, 22, 17, 30),
                                 'macroscopic': [],
                                 'redraws': {},
                                 'shapelessness': datetime.datetime(1966, 7, 22, 17, 30),
                                 'sting': {}},
                                True,
                                {'alley': {},
                                 'grouchy': True,
                                 'knees': {},
                                 'rakishly': 4758,
                                 'swashbuckling': 'chambers'},
                                datetime.datetime(1967, 3, 1, 13, 42),
                                True]],
                  'granules': ['litigates'],
                  'premiss': datetime.datetime(1995, 1, 23, 9, 59),
                  'tamed': {'Burgundian': 7837,
                            'Oceania': 'pedants',
                            'disuses': [True,
                                        9904,
                                        8025,
                                        {'exhilarate': True,
                                         'giveaways': [],
                                         'kneels': [],
                                         'poor': 'nationalists',
                                         'raglan': []}],
                            'packet': datetime.datetime(1912, 1, 9, 15, 4),
                            'verdict': datetime.datetime(2011, 11, 1, 16, 26)},
                  'therapeutic': 8212},
 'utilizes': {'Mitterrand': {'inclosing': datetime.datetime(1983, 6, 27, 3, 56),
                             'irately': datetime.datetime(1918, 3, 3, 8, 35),
                             'outsized': [{'Rx': {},
                                           'bare': 5115,
                                           'goats': 'Karakorum',
                                           'lessee': 'underlay',
                                           'typesets': 'judiciously'},
                                          False,
                                          {'concealment': 'distorting',
                                           'gracious': None,
                                           'host': [],
                                           'housemothers': True,
                                           'lamer': datetime.datetime(2011, 11, 1, 16, 26)}],
                             'rearmed': False,
                             'tallow': datetime.datetime(2013, 1, 22, 18, 59)},
              'hyphened': datetime.datetime(1911, 1, 17, 4, 24),
              'poverty': {'buoyancy': 7807,
                          'colorless': datetime.datetime(2005, 7, 13, 17, 18),
                          'finch': {'Meyers': {'anchorpersons': [],
                                               'breathe': [],
                                               'computational': True,
                                               'differentiation': datetime.datetime(1936, 12, 16, 10, 38),
                                               'warehouse': datetime.datetime(1966, 7, 22, 17, 30)},
                                    'arrives': False,
                                    'mausoleums': datetime.datetime(1966, 7, 22, 17, 30),
                                    'ponderous': 5684,
                                    'transmitted': True},
                          'mandarin': datetime.datetime(1930, 12, 12, 3, 58),
                          'touchy': datetime.datetime(1967, 3, 1, 13, 42)},
              'resurgences': datetime.datetime(1949, 6, 25, 23, 50),
              'whir': 'reenters'},
 'vichyssoise': [],
 'virus': {'Medicaids': 'gargle',
           'Pitt': 8044,
           'cormorant': 4672,
           'ministries': ['coffeecakes', True],
           'soy': 8861},
 'warred': datetime.datetime(1918, 3, 3, 8, 35),
 'weatherized': {'resilient': datetime.datetime(1966, 7, 22, 17, 30),
                 'snuggles': True,
                 'still': 'celesta',
                 'thefts': True,
                 'upliftings': {'Creator': datetime.datetime(2011, 11, 1, 16, 26),
                                'chevron': datetime.datetime(1930, 12, 12, 3, 58),
                                'mistranslated': datetime.datetime(1936, 12, 16, 10, 38),
                                'nationalized': datetime.datetime(1967, 3, 1, 13, 42),
                                'polygonal': [None, None, False, []]}},
 'whereof': 5169,
 'worry': 'rumples'}
